local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('sys_types')
lj_include('bits_uio')

ffi.cdef[[
struct flock {
    short int l_type;
    short int l_whence;
    __off_t l_start;
    __off_t l_len;
    __pid_t l_pid;
};
struct flock64 {
    short int l_type;
    short int l_whence;
    __off64_t l_start;
    __off64_t l_len;
    __pid_t l_pid;
};
ssize_t readahead(int __fd, __off64_t __offset, size_t __count);
int sync_file_range(int __fd, __off64_t __from, __off64_t __to,
    unsigned int __flags);
ssize_t vmsplice(int __fdout, const struct iovec *__iov,
    size_t __count, unsigned int __flags);
ssize_t splice(int __fdin, __off64_t * __offin, int __fdout,
    __off64_t * __offout, size_t __len, unsigned int __flags);
ssize_t tee(int __fdin, int __fdout, size_t __len, unsigned int __flags);
int fallocate(int __fd, int __mode, __off_t __offset, __off_t __len);
int fallocate64(int __fd, int __mode, __off64_t __offset, __off64_t __len);
]]

local defs = {
    O_ACCMODE = '0003',
    O_RDONLY = '00',
    O_WRONLY = '01',
    O_RDWR = '02',
    O_CREAT = '0100',
    O_EXCL = '0200',
    O_NOCTTY = '0400',
    O_TRUNC = '01000',
    O_APPEND = '02000',
    O_NONBLOCK = '04000',
    O_NDELAY = '04000',
    O_SYNC = '010000',
    O_FSYNC = '010000',
    O_ASYNC = '020000',
    O_DIRECT = '0400000',
    O_DIRECTORY = '040000',
    O_NOFOLLOW = '0100000',
    O_NOATIME = '01000000',
    O_CLOEXEC = '02000000',
    O_LARGEFILE = '0200000',
    O_DSYNC = '010000',
    O_RSYNC = '010000',
    F_DUPFD = '0',
    F_GETFD = '1',
    F_SETFD = '2',
    F_GETFL = '3',
    F_SETFL = '4',
    F_GETLK = '5',
    F_SETLK = '6',
    F_SETLKW = '7',
    F_GETLK64 = '12',
    F_SETLK64 = '13',
    F_SETLKW64 = '14',
    F_SETOWN = '8',
    F_GETOWN = '9',
    F_SETSIG = '10',
    F_GETSIG = '11',
    F_SETLEASE = '1024',
    F_GETLEASE = '1025',
    F_NOTIFY = '1026',
    F_DUPFD_CLOEXEC = '1030',
    FD_CLOEXEC = '1',
    F_RDLCK = '0',
    F_WRLCK = '1',
    F_UNLCK = '2',
    F_EXLCK = '4',
    F_SHLCK = '8',
    LOCK_SH = '1',
    LOCK_EX = '2',
    LOCK_NB = '4',
    LOCK_UN = '8',
    LOCK_MAND = '32',
    LOCK_READ = '64',
    LOCK_WRITE = '128',
    LOCK_RW = '192',
    DN_ACCESS = '0x00000001',
    DN_MODIFY = '0x00000002',
    DN_CREATE = '0x00000004',
    DN_DELETE = '0x00000008',
    DN_RENAME = '0x00000010',
    DN_ATTRIB = '0x00000020',
    DN_MULTISHOT = '0x80000000',
    FAPPEND = '02000',
    FFSYNC = '010000',
    FASYNC = '020000',
    FNONBLOCK = '04000',
    FNDELAY = '04000',
    POSIX_FADV_NORMAL = '0',
    POSIX_FADV_RANDOM = '1',
    POSIX_FADV_SEQUENTIAL = '2',
    POSIX_FADV_WILLNEED = '3',
    POSIX_FADV_DONTNEED = '4',
    POSIX_FADV_NOREUSE = '5',
    SYNC_FILE_RANGE_WAIT_BEFORE = '1',
    SYNC_FILE_RANGE_WRITE = '2',
    SYNC_FILE_RANGE_WAIT_AFTER = '4',
    SPLICE_F_MOVE = '1',
    SPLICE_F_NONBLOCK = '2',
    SPLICE_F_MORE = '4',
    SPLICE_F_GIFT = '8',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

