local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('sys_types')
lj_include('netinet_in')

ffi.cdef[[
struct timestamp {
    u_int8_t len;
    u_int8_t ptr;
    unsigned int overflow:4;
    unsigned int flags:4;
    u_int32_t data[9];
};
struct iphdr {
    unsigned int version:4;
    unsigned int ihl:4;
    u_int8_t tos;
    u_int16_t tot_len;
    u_int16_t id;
    u_int16_t frag_off;
    u_int8_t ttl;
    u_int8_t protocol;
    u_int16_t check;
    u_int32_t saddr;
    u_int32_t daddr;
};
struct ip {
    unsigned int ip_v:4;
    unsigned int ip_hl:4;
    u_int8_t ip_tos;
    u_short ip_len;
    u_short ip_id;
    u_short ip_off;
    u_int8_t ip_ttl;
    u_int8_t ip_p;
    u_short ip_sum;
    struct in_addr ip_src, ip_dst;
};
struct ip_timestamp {
    u_int8_t ipt_code;
    u_int8_t ipt_len;
    u_int8_t ipt_ptr;
    unsigned int ipt_oflw:4;
    unsigned int ipt_flg:4;
    u_int32_t data[9];
};
]]

local defs = {
    IP_RF = '0x8000',
    IP_DF = '0x4000',
    IP_MF = '0x2000',
    IP_OFFMASK = '0x1fff',
    IPVERSION = '4',
    IP_MAXPACKET = '65535',
    IPTOS_TOS_MASK = '0x1E',
    IPTOS_LOWDELAY = '0x10',
    IPTOS_THROUGHPUT = '0x08',
    IPTOS_RELIABILITY = '0x04',
    IPTOS_LOWCOST = '0x02',
    IPTOS_MINCOST = '0x02',
    IPTOS_PREC_MASK = '0xe0',
    IPTOS_PREC_NETCONTROL = '0xe0',
    IPTOS_PREC_INTERNETCONTROL = '0xc0',
    IPTOS_PREC_CRITIC_ECP = '0xa0',
    IPTOS_PREC_FLASHOVERRIDE = '0x80',
    IPTOS_PREC_FLASH = '0x60',
    IPTOS_PREC_IMMEDIATE = '0x40',
    IPTOS_PREC_PRIORITY = '0x20',
    IPTOS_PREC_ROUTINE = '0x00',
    IPOPT_COPY = '0x80',
    IPOPT_CLASS_MASK = '0x60',
    IPOPT_NUMBER_MASK = '0x1f',
    IPOPT_CONTROL = '0x00',
    IPOPT_RESERVED1 = '0x20',
    IPOPT_DEBMEAS = '0x40',
    IPOPT_MEASUREMENT = '0x40',
    IPOPT_RESERVED2 = '0x60',
    IPOPT_EOL = '0',
    IPOPT_END = '0',
    IPOPT_NOP = '1',
    IPOPT_NOOP = '1',
    IPOPT_RR = '7',
    IPOPT_TS = '68',
    IPOPT_TIMESTAMP = '68',
    IPOPT_SECURITY = '130',
    IPOPT_SEC = '130',
    IPOPT_LSRR = '131',
    IPOPT_SATID = '136',
    IPOPT_SID = '136',
    IPOPT_SSRR = '137',
    IPOPT_RA = '148',
    IPOPT_OPTVAL = '0',
    IPOPT_OLEN = '1',
    IPOPT_OFFSET = '2',
    IPOPT_MINOFF = '4',
    MAX_IPOPTLEN = '40',
    IPOPT_TS_TSONLY = '0',
    IPOPT_TS_TSANDADDR = '1',
    IPOPT_TS_PRESPEC = '3',
    IPOPT_SECUR_UNCLASS = '0x0000',
    IPOPT_SECUR_CONFID = '0xf135',
    IPOPT_SECUR_EFTO = '0x789a',
    IPOPT_SECUR_MMMM = '0xbc4d',
    IPOPT_SECUR_RESTR = '0xaf13',
    IPOPT_SECUR_SECRET = '0xd788',
    IPOPT_SECUR_TOPSECRET = '0x6bc5',
    MAXTTL = '255',
    IPDEFTTL = '64',
    IPFRAGTTL = '60',
    IPTTLDEC = '1',
    IP_MSS = '576',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

