local ffi = require'ptpv2-ffi/cdata'
local C = ffi.C

local m = {}

local function tx_ts_cb(ts, raw)
end

function m:send(time)
    local ci = self.clock_info
    local size = ci.sizeof_followup_msg
    
    ffi.fill(ci.ibuf, size + 2)
    self:build_msg_header()
    local msg = ffi.cast('followup_msg_t *', ci.ibuf)
    msg.hdr.transport_specific_message_type = C.PTP_MSG_TYPE_FOLLOW_UP
    msg.hdr.message_length = C.htons(size)
    msg.hdr.sequence_id = C.htons(ci.sent_sync_sequence_id - 1)
    msg.hdr.control_field = 0x2
    msg.hdr.log_message_interval = ci.log_sync_interval

    local pot = time:to_timestamp()
    msg.precise_origin_timestamp.seconds.msb = C.ntohs(pot.seconds.msb)
    msg.precise_origin_timestamp.seconds.lsb = C.ntohl(pot.seconds.lsb)
    msg.precise_origin_timestamp.nanoseconds = C.ntohl(pot.nanoseconds)

    if ci.transport_is_ipv6 ~= 0 then
        size = size + 2
    end

    local sock = self[C.GENERAL_PORT]
    local r, e = pcall(sock.sendto, sock, ci.ibuf, size, tx_ts_cb)
    if not r then
        self:log("error", string.format("send_followup(): sendto() failed - %s",
            e))
        self:to_state(C.PTP_STATE_FAULTY)
        return
    end
end

function m:faulty()
    -- FIXME: Try to recover from faulty state
end

function m:uncalibrated(time, from_me)
    if self:is_from_current_master() then
        local ci = self.clock_info
        local hdr = ffi.cast('msg_hdr_t *', ci.ibuf)
        local msg = ffi.cast('followup_msg_t *', ci.ibuf)
        if ci.waiting_for_follow ~= 0 and
            ci.recv_sync_sequence_id == hdr.sequence_id then
            ci.waiting_for_follow = 0
            local pot =
                self.timing_iface:to_internal_time(msg.precise_origin_timestamp)
            local cf =
                self.timing_iface:i64_to_internal_time(hdr.correction_field) +
                    self.last_sync_correction_field
            self:update_offset(pot, cf)
            self:update_clock()
        elseif ci.waiting_for_follow == 0 then
            -- Possibly out-of-order followup message
            ci.oo_followup = 1
            ffi.copy(ci.oo_fu_msg, ci.ibuf, ffi.sizeof('followup_msg_t'))
        else
            self:log("warning",
                string.format("Followup msg seq id mismatch: %d -> %d",
                    ci.recv_sync_sequence_id, hdr.sequence_id))
        end
    end
end

m.handle_state = {}
m.handle_state[C.PTP_STATE_FAULTY] = m.faulty
m.handle_state[C.PTP_STATE_UNCALIBRATED] = m.uncalibrated
m.handle_state[C.PTP_STATE_SLAVE] = m.handle_state[C.PTP_STATE_UNCALIBRATED]

function m:handle(time, from_me)
    local ci = self.clock_info
    local state = ci.state
    if from_me then
        return
    end
    local msg = ffi.cast('followup_msg_t *', ci.ibuf)
    msg.precise_origin_timestamp.seconds.msb =
        C.ntohs(msg.precise_origin_timestamp.seconds.msb)
    msg.precise_origin_timestamp.seconds.lsb =
        C.ntohl(msg.precise_origin_timestamp.seconds.lsb)
    msg.precise_origin_timestamp.nanoseconds =
        C.ntohl(msg.precise_origin_timestamp.nanoseconds)
    local f = m.handle_state[state]
    if f then
        f(self, time, from_me)
    end
end

return m
