local alien = require 'alien'
alien.struct = require 'alien.struct'

local sie = { }
local s = alien.load('libsie.so.0')

local function wrap(name, ...)
    s[name]:types(...)
    local sname = assert(name:match('^sie_(.*)'))
    sie[sname] = function(...) return s[name](...) end
end

wrap('sie_release', 'void', 'pointer')

s.sie_iterator_next:types('pointer', 'pointer')
local function wrap_iter(name)
    s[name]:types('pointer', 'pointer')
    local function mkiter(iter)
        iter = s[name](iter)
        local function iterfn(state, var1)
            var1 = s.sie_iterator_next(iter)
            if not var1 then
                sie.release(iter)
            end
            return var1
        end
        return iterfn, nil, nil
    end
    local sname = assert(name:match('^sie_(.*)'))
    sie[sname] = mkiter
end

wrap('sie_context_new', 'pointer')
wrap('sie_context_done', 'int', 'pointer')

wrap('sie_stream_new', 'pointer', 'pointer')
s.sie_add_stream_data:types('ulong', 'pointer', 'string', 'ulong')
function sie.add_stream_data(stream, data)
    return s.sie_add_stream_data(stream, data, data:len())
end

wrap_iter('sie_get_tests')
wrap_iter('sie_get_channels')
wrap_iter('sie_get_dimensions')
wrap_iter('sie_get_tags')

wrap('sie_get_test', 'pointer', 'pointer', 'uint')
wrap('sie_get_channel', 'pointer', 'pointer', 'uint')
wrap('sie_get_dimension', 'pointer', 'pointer', 'uint')
wrap('sie_get_tag', 'pointer', 'pointer', 'string')
wrap('sie_get_id', 'uint', 'pointer')
wrap('sie_get_name', 'string', 'pointer')

wrap('sie_free', 'void', 'pointer')
wrap('sie_tag_get_id', 'string', 'pointer')
s.sie_tag_get_value:types('pointer', 'pointer')
function sie.tag_get_value(tag)
    local val = s.sie_tag_get_value(tag)
    local str = alien.tostring(val)
    sie.free(val)
    return str
end

wrap('sie_attach_spigot', 'pointer', 'pointer')
wrap('sie_spigot_get', 'pointer', 'pointer')
wrap('sie_spigot_done', 'int', 'pointer')

wrap('sie_output_get_num_dims', 'ulong', 'pointer')
wrap('sie_output_get_num_rows', 'ulong', 'pointer')
s.sie_output_get_float64:types('pointer', 'pointer', 'ulong')
function sie.output_get_float64(output, dim)
    local arr = s.sie_output_get_float64(output, dim)
    local nrows = sie.output_get_num_rows(output)
    local str = alien.tostring(arr, nrows * 8)
    local ret = { }
    for i=1,nrows do
        local o = i - 1
        local s = str:sub(1 + o * 8, 8 + o * 8)
        local d = alien.struct.unpack('d', s)
        table.insert(ret, d)
    end
    return ret
end
s.sie_output_get_raw:types('pointer', 'pointer', 'ulong')
function sie.output_get_raw(output, dim)
    local arr = s.sie_output_get_raw(output, dim)
    local nrows = sie.output_get_num_rows(output)
    local str = alien.tostring(arr, nrows * 12)
    local ret = { }
    for i=1,nrows do
        local o = i - 1
        local s = str:sub(1 + o * 12, 12 + o * 12)
        local ptr, size = alien.struct.unpack('pL', s)
        table.insert(ret, alien.tostring(ptr, size))
    end
    return ret
end

return sie
